@@ -13,6 +13,7 @@ import signal
 import sys
 import time
 import uuid
+import warnings
 from enum import Enum
 from logging import getLogger
 from threading import Lock, Timer
@@ -71,6 +72,7 @@ from .time_util import get_time_millis
 
 if TYPE_CHECKING:  # pragma: no cover
     from pandas import DataFrame
+    from pyarrow import Table
 
     from .connection import SnowflakeConnection
     from .file_transfer_agent import SnowflakeProgressPercentage
@@ -80,14 +82,17 @@ T = TypeVar("T", bound=collections.abc.Sequence)
 
 logger = getLogger(__name__)
 
-if installed_pandas:
-    from pyarrow import Table
-else:
-    logger.debug("Failed to import pyarrow. Cannot use pandas fetch API")
-    Table = None
+
+if not installed_pandas:
+    logger.debug(
+        "Failed to import pyarrow or pandas. Cannot use pandas fetch API. Please "
+        "install snowflake-connector-python with the `pandas` extra to use these "
+        "features."
+    )
+
 
 try:
-    from .arrow_iterator import PyArrowIterator  # NOQA
+    from .nanoarrow_arrow_iterator import PyArrowIterator  # NOQA
 
     CAN_USE_ARROW_RESULT_FORMAT = True
 except ImportError as e:  # pragma: no cover
@@ -122,7 +127,7 @@ ASYNC_NO_DATA_MAX_RETRY = 24
 ASYNC_RETRY_PATTERN = [1, 1, 2, 3, 4, 8, 10]
 
 
-class NanoarrowUsage(str, Enum):
+class _NanoarrowUsage(str, Enum):
     # follow the session parameter to use nanoarrow converter or not
     FOLLOW_SESSION_PARAMETER = "follow_session_parameter"
     # ignore the session parameter, use nanoarrow converter
@@ -131,25 +136,6 @@ class NanoarrowUsage(str, Enum):
     DISABLE_NANOARROW = "disable_nanoarrow"
 
 
-def _get_client_nanoarrow_setting() -> NanoarrowUsage:
-    ret = os.environ.get("NANOARROW_USAGE", NanoarrowUsage.FOLLOW_SESSION_PARAMETER)
-    if ret is not NanoarrowUsage.FOLLOW_SESSION_PARAMETER:
-        try:
-            ret = NanoarrowUsage(ret.lower())
-        except ValueError:
-            logger.warning(
-                "Invalid value for enum NanoarrowUsage, valid options are"
-                "FOLLOW_SESSION_PARAMETER, ENABLE_NANOARROW, DISABLE_NANOARROW,"
-                "falling back to use NanoarrowUsage.FOLLOW_SESSION_PARAMETER"
-            )
-            ret = NanoarrowUsage.FOLLOW_SESSION_PARAMETER
-    return ret
-
-
-# client parameter
-NANOARROW_USAGE = _get_client_nanoarrow_setting()
-
-
 class ResultMetadata(NamedTuple):
     name: str
     type_code: int
@@ -162,13 +148,15 @@ class ResultMetadata(NamedTuple):
     @classmethod
     def from_column(cls, col: dict[str, Any]):
         """Initializes a ResultMetadata object from the column description in the query response."""
+        type_code = FIELD_NAME_TO_ID[
+            col["extTypeName"].upper()
+            if col.get("extTypeName")
+            else col["type"].upper()
+        ]
+
         return cls(
             col["name"],
-            FIELD_NAME_TO_ID[
-                col["extTypeName"].upper()
-                if col.get("extTypeName")
-                else col["type"].upper()
-            ],
+            type_code,
             None,
             col["length"],
             col["precision"],
@@ -177,6 +165,205 @@ class ResultMetadata(NamedTuple):
         )
 
 
+class ResultMetadataV2Field:
+    """ResultMetadataV2Field represents the type information of one sub-type in a nested type."""
+
+    def __init__(
+        self,
+        type_code: int,
+        is_nullable: bool,
+        internal_size: int | None = None,
+        precision: int | None = None,
+        scale: int | None = None,
+        fields: list[ResultMetadataV2] | None = None,
+    ):
+        self._type_code = type_code
+        self._is_nullable = is_nullable
+        self._internal_size = internal_size
+        self._precision = precision
+        self._scale = scale
+        self._fields = fields
+
+    @classmethod
+    def from_column(cls, col: dict[str, Any]) -> ResultMetadataV2Field:
+        """Initializes a ResultMetadataV2Field object from a child of the column description in the query response."""
+        fields = None
+        if col.get("fields") is not None:
+            fields = [cls.from_column(f) for f in col["fields"]]
+        return cls(
+            FIELD_NAME_TO_ID[
+                col["extTypeName"].upper()
+                if col.get("extTypeName")
+                else col["type"].upper()
+            ],
+            col["nullable"],
+            col["length"],
+            col["precision"],
+            col["scale"],
+            fields,
+        )
+
+    @property
+    def type_code(self) -> int:
+        return self._type_code
+
+    @property
+    def is_nullable(self) -> bool:
+        return self._is_nullable
+
+    @property
+    def internal_size(self) -> int | None:
+        return self._internal_size
+
+    @property
+    def precision(self) -> int | None:
+        return self._precision
+
+    @property
+    def scale(self) -> int | None:
+        return self._scale
+
+    @property
+    def fields(self) -> list[ResultMetadataV2Field] | None:
+        return self._fields
+
+
+class ResultMetadataV2:
+    """ResultMetadataV2Field represents the type information of a single column.
+
+    It is a replacement for ResultMetadata that contains additional attributes, currently
+    `vector_dimension` and `fields`. This class will be unified with ResultMetadata in the
+    near future.
+    """
+
+    def __init__(
+        self,
+        name: str,
+        type_code: int,
+        is_nullable: bool,
+        display_size: int | None = None,
+        internal_size: int | None = None,
+        precision: int | None = None,
+        scale: int | None = None,
+        vector_dimension: int | None = None,
+        fields: list[ResultMetadataV2] | None = None,
+    ):
+        self._name = name
+        self._type_code = type_code
+        self._is_nullable = is_nullable
+        self._display_size = display_size
+        self._internal_size = internal_size
+        self._precision = precision
+        self._scale = scale
+        self._vector_dimension = vector_dimension
+        self._fields = fields
+
+    @classmethod
+    def from_column(cls, col: dict[str, Any]) -> ResultMetadataV2:
+        """Initializes a ResultMetadataV2 object from the column description in the query response.
+        This differs from ResultMetadata in that it has newly-added fields which cannot be added to
+        ResultMetadata since it is a named tuple.
+        """
+        type_code = FIELD_NAME_TO_ID[
+            col["extTypeName"].upper()
+            if col.get("extTypeName")
+            else col["type"].upper()
+        ]
+
+        fields = None
+        if type_code == FIELD_NAME_TO_ID["VECTOR"] and col.get("fields") is not None:
+            fields = [ResultMetadataV2Field.from_column(f) for f in col["fields"]]
+
+        return cls(
+            col["name"],
+            type_code,
+            col["nullable"],
+            None,
+            col["length"],
+            col["precision"],
+            col["scale"],
+            col.get("vectorDimension"),
+            fields,
+        )
+
+    def _to_result_metadata_v1(self):
+        """Initializes a ResultMetadata object from a ResultMetadataV2 object.
+
+        This method is for internal use only.
+        """
+
+        return ResultMetadata(
+            self._name,
+            self._type_code,
+            self._display_size,
+            self._internal_size,
+            self._precision,
+            self._scale,
+            self._is_nullable,
+        )
+
+    def __str__(self) -> str:
+        return (
+            f"ResultMetadataV2(name={self._name},type_code={self._type_code},"
+            + f"is_nullable={self._is_nullable},display_size={self._display_size},"
+            + "internal_size={self._internal_size},precision={self._precision},"
+            + "scale={self._scale},vector_dimension={self._vector_dimension},"
+            + "fields={self.fields})"
+        )
+
+    def __eq__(self, other) -> bool:
+        if not isinstance(other, self.__class__):
+            return False
+
+        return (
+            self._name == other._name
+            and self._type_code == other._type_code
+            and self._is_nullable == other._is_nullable
+            and self._display_size == other._display_size
+            and self._internal_size == other._internal_size
+            and self._precision == other._precision
+            and self._scale == other._scale
+            and self._vector_dimension == other._vector_dimension
+            and self._fields == other._fields
+        )
+
+    @property
+    def name(self) -> str:
+        return self._name
+
+    @property
+    def type_code(self) -> int:
+        return self._type_code
+
+    @property
+    def is_nullable(self) -> bool:
+        return self._is_nullable
+
+    @property
+    def internal_size(self) -> int | None:
+        return self._internal_size
+
+    @property
+    def display_size(self) -> int | None:
+        return self._display_size
+
+    @property
+    def precision(self) -> int | None:
+        return self._precision
+
+    @property
+    def scale(self) -> int | None:
+        return self._scale
+
+    @property
+    def vector_dimension(self) -> int | None:
+        return self._vector_dimension
+
+    @property
+    def fields(self) -> list[ResultMetadataV2Field] | None:
+        return self._fields
+
+
 def exit_handler(*_) -> NoReturn:
     """Handler for signal. When called, it will raise SystemExit with exit code FORCE_EXIT."""
     print("\nForce exit")
@@ -258,7 +445,7 @@ class SnowflakeCursor:
             tuple[type[Error] | type[Exception], dict[str, str | bool]]
         ] = []
         self._timebomb: Timer | None = None  # must be here for abort_exit method
-        self._description: list[ResultMetadata] | None = None
+        self._description: list[ResultMetadataV2] | None = None
         self._sfqid: str | None = None
         self._sqlstate = None
         self._total_rowcount = -1
@@ -306,6 +493,17 @@ class SnowflakeCursor:
 
     @property
     def description(self) -> list[ResultMetadata]:
+        if self._description is None:
+            return None
+
+        return [meta._to_result_metadata_v1() for meta in self._description]
+
+    @property
+    def _description_internal(self) -> list[ResultMetadataV2]:
+        """Return the new format of result metadata for a query.
+
+        This method is for internal use only.
+        """
         return self._description
 
     @property
@@ -958,6 +1156,24 @@ class SnowflakeCursor:
         """
         kwargs["_describe_only"] = kwargs["_is_internal"] = True
         self.execute(*args, **kwargs)
+
+        if self._description is None:
+            return None
+        return [meta._to_result_metadata_v1() for meta in self._description]
+
+    def _describe_internal(self, *args: Any, **kwargs: Any) -> list[ResultMetadataV2]:
+        """Obtain the schema of the result without executing the query.
+
+        This function takes the same arguments as execute, please refer to that function
+        for documentation.
+
+        This function is for internal use only
+
+        Returns:
+            The schema of the result, in the new result metadata format.
+        """
+        kwargs["_describe_only"] = kwargs["_is_internal"] = True
+        self.execute(*args, **kwargs)
         return self._description
 
     def _format_query_for_log(self, query: str) -> str:
@@ -977,8 +1193,8 @@ class SnowflakeCursor:
         if self._total_rowcount == -1 and not is_dml and data.get("total") is not None:
             self._total_rowcount = data["total"]
 
-        self._description: list[ResultMetadata] = [
-            ResultMetadata.from_column(col) for col in data["rowtype"]
+        self._description: list[ResultMetadataV2] = [
+            ResultMetadataV2.from_column(col) for col in data["rowtype"]
         ]
 
         result_chunks = create_batches_from_response(
@@ -1002,11 +1218,11 @@ class SnowflakeCursor:
         if is_dml and "rowset" in data and len(data["rowset"]) > 0:
             updated_rows = 0
             for idx, desc in enumerate(self._description):
-                if desc[0] in (
+                if desc.name in (
                     "number of rows updated",
                     "number of multi-joined rows updated",
                     "number of rows deleted",
-                ) or desc[0].startswith("number of rows inserted"):
+                ) or desc.name.startswith("number of rows inserted"):
                     updated_rows += int(data["rowset"][0][idx])
             if self._total_rowcount == -1:
                 self._total_rowcount = updated_rows
@@ -1117,14 +1333,29 @@ class SnowflakeCursor:
         )
         return self._result_set._fetch_arrow_batches()
 
-    def fetch_arrow_all(self) -> Table | None:
+    @overload
+    def fetch_arrow_all(self, force_return_table: Literal[False]) -> Table | None:
+        ...
+
+    @overload
+    def fetch_arrow_all(self, force_return_table: Literal[True]) -> Table:
+        ...
+
+    def fetch_arrow_all(self, force_return_table: bool = False) -> Table | None:
+        """
+        Args:
+            force_return_table: Set to True so that when the query returns zero rows,
+                an empty pyarrow table will be returned with schema using the highest bit length for each column.
+                Default value is False in which case None is returned in case of zero rows.
+        """
         self.check_can_use_arrow_resultset()
+
         if self._prefetch_hook is not None:
             self._prefetch_hook()
         if self._query_result_format != "arrow":
             raise NotSupportedError
         self._log_telemetry_job_data(TelemetryField.ARROW_FETCH_ALL, TelemetryData.TRUE)
-        return self._result_set._fetch_arrow_all()
+        return self._result_set._fetch_arrow_all(force_return_table=force_return_table)
 
     def fetch_pandas_batches(self, **kwargs: Any) -> Iterator[DataFrame]:
         """Fetches a single Arrow Table."""
@@ -1524,3 +1755,31 @@ class DictCursor(SnowflakeCursor):
             connection,
             use_dict_result=True,
         )
+
+
+def __getattr__(name):
+    if name == "NanoarrowUsage":
+        warnings.warn(
+            "snowflake.connector.cursor.NanoarrowUsage has been deprecated and will be removed in the future, "
+            "please stop using the class.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        return _NanoarrowUsage
+    if name == "NANOARROW_USAGE":
+        if "NANOARROW_USAGE" in os.environ:
+            warnings.warn(
+                "Environment variable NANOARROW_USAGE has been deprecated and will be removed in the future, "
+                "please stop setting the variable.",
+                DeprecationWarning,
+                stacklevel=2,
+            )
+
+        warnings.warn(
+            "snowflake.connector.cursor.NANOARROW_USAGE has been deprecated and will be removed in the future, "
+            "please stop using the variable.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        return None
+    raise AttributeError(f"module {__name__!r} has no attribute {name!r}")
