@@ -5,7 +5,6 @@
 from __future__ import annotations
 
 import abc
-import io
 import json
 import time
 from base64 import b64decode
@@ -43,12 +42,12 @@ if TYPE_CHECKING:  # pragma: no cover
 
     from .connection import SnowflakeConnection
     from .converter import SnowflakeConverterType
-    from .cursor import ResultMetadata, SnowflakeCursor
+    from .cursor import ResultMetadataV2, SnowflakeCursor
     from .vendored.requests import Response
 
 
 # emtpy pyarrow type array corresponding to FIELD_TYPES
-FIELD_TYPE_TO_PA_TYPE: list[DataType] = []
+FIELD_TYPE_TO_PA_TYPE: list[Callable[[ResultMetadataV2], DataType]] = []
 
 # qrmk related constants
 SSE_C_ALGORITHM = "x-amz-server-side-encryption-customer-algorithm"
@@ -56,30 +55,6 @@ SSE_C_KEY = "x-amz-server-side-encryption-customer-key"
 SSE_C_AES = "AES256"
 
 
-def _create_vendored_arrow_iterator(
-    data: bytes,
-    context: ArrowConverterContext,
-    use_dict_result: bool,
-    numpy: bool,
-    number_to_decimal: bool,
-    row_unit: IterUnit,
-):
-    from .arrow_iterator import PyArrowIterator
-
-    logger.debug("Using vendored arrow as the arrow data converter")
-    iter = PyArrowIterator(
-        None,
-        io.BytesIO(data),
-        context,
-        use_dict_result,
-        numpy,
-        number_to_decimal,
-    )
-    if row_unit == IterUnit.TABLE_UNIT:
-        iter.init_table_unit()
-    return iter
-
-
 def _create_nanoarrow_iterator(
     data: bytes,
     context: ArrowConverterContext,
@@ -133,7 +108,7 @@ def create_batches_from_response(
     cursor: SnowflakeCursor,
     _format: str,
     data: dict[str, Any],
-    schema: Sequence[ResultMetadata],
+    schema: Sequence[ResultMetadataV2],
 ) -> list[ResultBatch]:
     column_converters: list[tuple[str, SnowflakeConverterType]] = []
     arrow_context: ArrowConverterContext | None = None
@@ -204,7 +179,6 @@ def create_batches_from_response(
                     cursor._connection._numpy,
                     schema,
                     cursor._connection._arrow_number_to_decimal,
-                    cursor._connection._create_pyarrow_iterator_method,
                 )
                 for c in chunks
             ]
@@ -227,7 +201,6 @@ def create_batches_from_response(
             cursor._connection._numpy,
             schema,
             cursor._connection._arrow_number_to_decimal,
-            cursor._connection._create_pyarrow_iterator_method,
         )
     else:
         logger.error(f"Don't know how to construct ResultBatches from response: {data}")
@@ -239,7 +212,6 @@ def create_batches_from_response(
             cursor._connection._numpy,
             schema,
             cursor._connection._arrow_number_to_decimal,
-            cursor._connection._create_pyarrow_iterator_method,
         )
 
     return [first_chunk] + rest_of_chunks
@@ -271,13 +243,16 @@ class ResultBatch(abc.ABC):
         rowcount: int,
         chunk_headers: dict[str, str] | None,
         remote_chunk_info: RemoteChunkInfo | None,
-        schema: Sequence[ResultMetadata],
+        schema: Sequence[ResultMetadataV2],
         use_dict_result: bool,
     ) -> None:
         self.rowcount = rowcount
         self._chunk_headers = chunk_headers
         self._remote_chunk_info = remote_chunk_info
-        self.schema = schema
+        self._schema = schema
+        self.schema = (
+            [s._to_result_metadata_v1() for s in schema] if schema is not None else None
+        )
         self._use_dict_result = use_dict_result
         self._metrics: dict[str, int] = {}
         self._data: str | list[tuple[Any, ...]] | None = None
@@ -315,7 +290,7 @@ class ResultBatch(abc.ABC):
 
     @property
     def column_names(self) -> list[str]:
-        return [col.name for col in self.schema]
+        return [col.name for col in self._schema]
 
     def __iter__(
         self,
@@ -446,7 +421,7 @@ class JSONResultBatch(ResultBatch):
         rowcount: int,
         chunk_headers: dict[str, str] | None,
         remote_chunk_info: RemoteChunkInfo | None,
-        schema: Sequence[ResultMetadata],
+        schema: Sequence[ResultMetadataV2],
         column_converters: Sequence[tuple[str, SnowflakeConverterType]],
         use_dict_result: bool,
         *,
@@ -467,7 +442,7 @@ class JSONResultBatch(ResultBatch):
         cls,
         data: Sequence[Sequence[Any]],
         data_len: int,
-        schema: Sequence[ResultMetadata],
+        schema: Sequence[ResultMetadataV2],
         column_converters: Sequence[tuple[str, SnowflakeConverterType]],
         use_dict_result: bool,
     ):
@@ -518,7 +493,7 @@ class JSONResultBatch(ResultBatch):
                     for (_t, c), v, col in zip(
                         self.column_converters,
                         row,
-                        self.schema,
+                        self._schema,
                     ):
                         row_result[col.name] = v if c is None or v is None else c(v)
                     result_list.append(row_result)
@@ -536,13 +511,13 @@ class JSONResultBatch(ResultBatch):
                     )
         else:
             for row in downloaded_data:
-                row_result = [None] * len(self.schema)
+                row_result = [None] * len(self._schema)
                 try:
                     idx = 0
                     for (_t, c), v, _col in zip(
                         self.column_converters,
                         row,
-                        self.schema,
+                        self._schema,
                     ):
                         row_result[idx] = v if c is None or v is None else c(v)
                         idx += 1
@@ -604,9 +579,8 @@ class ArrowResultBatch(ResultBatch):
         context: ArrowConverterContext,
         use_dict_result: bool,
         numpy: bool,
-        schema: Sequence[ResultMetadata],
+        schema: Sequence[ResultMetadataV2],
         number_to_decimal: bool,
-        create_pyarrow_iterator_method: Callable,
     ) -> None:
         super().__init__(
             rowcount,
@@ -618,7 +592,6 @@ class ArrowResultBatch(ResultBatch):
         self._context = context
         self._numpy = numpy
         self._number_to_decimal = number_to_decimal
-        self._create_pyarrow_iterator_method = create_pyarrow_iterator_method
 
     def __repr__(self) -> str:
         return f"ArrowResultChunk({self.id})"
@@ -631,7 +604,7 @@ class ArrowResultBatch(ResultBatch):
         This is used to iterate through results in different ways depending on which
         mode that ``PyArrowIterator`` is in.
         """
-        return self._create_pyarrow_iterator_method(
+        return _create_nanoarrow_iterator(
             response.content,
             self._context,
             self._use_dict_result,
@@ -651,7 +624,7 @@ class ArrowResultBatch(ResultBatch):
         if len(data) == 0:
             return iter([])
 
-        return self._create_pyarrow_iterator_method(
+        return _create_nanoarrow_iterator(
             b64decode(data),
             self._context,
             self._use_dict_result,
@@ -668,9 +641,8 @@ class ArrowResultBatch(ResultBatch):
         context: ArrowConverterContext,
         use_dict_result: bool,
         numpy: bool,
-        schema: Sequence[ResultMetadata],
+        schema: Sequence[ResultMetadataV2],
         number_to_decimal: bool,
-        create_pyarrow_iterator_method: Callable,
     ):
         """Initializes an ``ArrowResultBatch`` from static, local data."""
         new_chunk = cls(
@@ -682,7 +654,6 @@ class ArrowResultBatch(ResultBatch):
             numpy,
             schema,
             number_to_decimal,
-            create_pyarrow_iterator_method,
         )
         new_chunk._data = data
 
@@ -712,9 +683,10 @@ class ArrowResultBatch(ResultBatch):
         """Returns empty Arrow table based on schema"""
         if installed_pandas:
             # initialize pyarrow type array corresponding to FIELD_TYPES
-            FIELD_TYPE_TO_PA_TYPE = [e.pa_type() for e in FIELD_TYPES]
+            FIELD_TYPE_TO_PA_TYPE = [e.pa_type for e in FIELD_TYPES]
         fields = [
-            pa.field(s.name, FIELD_TYPE_TO_PA_TYPE[s.type_code]) for s in self.schema
+            pa.field(s.name, FIELD_TYPE_TO_PA_TYPE[s.type_code](s))
+            for s in self._schema
         ]
         return pa.schema(fields).empty_table()
 
